unit CalibrationUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    ADTime: TTimer;
    MainPanel: TPanel;
    MainMemo: TMemo;
    Panel2: TPanel;
    Label1: TLabel;
    GoButton: TButton;
    RangeRadios: TRadioGroup;
    OffsetEdit: TEdit;
    ChannelPanel: TPanel;
    ADPanel: TPanel;
    ADCheck: TCheckBox;
    ADPaint: TPaintBox;
    AOPanel: TPanel;
    AOADPanel: TPanel;
    Label3: TLabel;
    SmoothTrack: TTrackBar;
    AOADLabel: TLabel;
    AODAPanel: TPanel;
    DACHighButton: TButton;
    DACMidButton: TButton;
    DACLowButton: TButton;
    Label2: TLabel;
    Button1: TButton;
    DelayPanel: TPanel;
    procedure GoButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ADCheckClick(Sender: TObject);
    procedure ADTimeTimer(Sender: TObject);
    procedure DACLowButtonClick(Sender: TObject);
    procedure DACMidButtonClick(Sender: TObject);
    procedure DACHighButtonClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure DelayPanelClick(Sender: TObject);
  protected
    procedure DoReportStatus(Wot: String; Status: LongWord);
    property ReportStatus[Wot: String]: LongWord write DoReportStatus;

    procedure DelayTrackChange(Sender: TObject);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses
  Math, Registry, AIOUSB;

{$R *.DFM}

type
  TUSBAI1616Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //added for compatibility with the AI-MUX family
  end;

function FloatToHex(F: Double): String;
var
  I, Whole: Integer;
  bNeg: Boolean;

begin
  bNeg := F < 0;
  if bNeg then F := -F;

  Whole := Floor(F);
  F := F - Whole;
  Result := IntToHex(Whole, 4) + '.';

  for I := 1 to 3 do begin
    F := F * 16;
    Whole := Floor(F);
    F := F - Whole;
    Result := Result + IntToHex(Whole, 1);
  end;
  Result := Result + IntToHex(Round(F * 16), 1);

  if bNeg then Result := '-' + Result;
end;

function MyFloatToStr(F: Double): String;
var
  I, V, VDigit: Integer;
  bNegative: Boolean;

begin
  bNegative := F < 0;
  V := Abs(Round(1000 * F));
  Result := '';
  for I := 0 to 2 do begin
    VDigit := V mod 10;
    V := V div 10;
    Result := Char(Byte('0') + VDigit) + Result;
  end;
  Result := '.' + Result;
  if V <> 0 then Result := IntToStr(V) + Result;
  if bNegative then Result := '-' + Result;
end;

function StrFromSecQuarters(Quarters: Integer): AnsiString;
begin
  case Quarters of
    1, 2, 3: Result := '';
    else Result := IntToStr(Quarters div 4);
  end;
  case Quarters mod 4 of
    1: Result := Result + '';
    2: Result := Result + '';
    3: Result := Result + '';
  end;
  Result := Result + ' sec';
end;

function FindChildControlAndIMeanControl(ParentControl: TWinControl; const ControlName: string): TControl;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to ParentControl.ControlCount - 1 do
    if CompareText(ParentControl.Controls[I].Name, ControlName) = 0 then begin
      Result := ParentControl.Controls[I];
      Exit;
    end
  ;
end;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

const
  MyKey = 'Software\ACCES\AOCal\';

var
  TicksPerSec, TicksPerMillisec: Int64;
  AIDI, AODI: LongWord;
  Config: TUSBAI1616Config;
  Span: Integer;
  BiOff, OffCt: Double;
  AOChannels: Integer;

  ChannelCheck: array[0..15] of TCheckBox;
  FullADHeight, FullAOHeight: Integer;

  InterChannelDelay: Integer = 0;

procedure ProcessDelay(Milliseconds: Integer);
var
  EndTime, Naui: Int64;

begin
  QueryPerformanceCounter(EndTime);
  EndTime := EndTime + TicksPerMillisec * Milliseconds;
  repeat
    Application.ProcessMessages;
    //if bCancel then Exit;

    QueryPerformanceCounter(Naui);
    Naui := Naui - EndTime;
  until Naui > 0;
end;

function SetAllDACs(Counts: Word): LongWord;
var
  DABuf: array[0..31] of Word;
  Ch: Integer;

begin
  for Ch := 0 to 15 do begin
    DABuf[Ch * 2] := Ch;
    DABuf[Ch * 2 + 1] := Counts;
  end;
  Result := DACMultiDirect(AODI, @DABuf[0], AOChannels);
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
const
  LoRef = $1000;
  HiRef = $F000;
  NybbleToHex: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

var
  ConfigBufSize: LongWord;
  ADLo, ADHi: array[0..15] of Double;
  I, Ch: Integer;
  Status, PID, TC: LongWord;
  TarEntry: Double;
  CalEntry: Word;
  MemoBuf: String;
  CalBuf, FilBuf: String;
  Fil: TFileStream;
  CheckChannel: array of Boolean;
  AIRange, AORange: Byte;
  bAOHasAD: Boolean;
  CalCheck: array of Word;
  Offset, L, CalCheckSize: LongWord;

    (*
    procedure SetAllDACs(Counts: Word);
    var
      DABuf: array[0..31] of Word;
      Ch: Integer;

    begin
      //{
      for Ch := 0 to 15 do begin
        DABuf[Ch * 2] := Ch;
        DABuf[Ch * 2 + 1] := Counts;
      end;
      ReportStatus['DACMultiDirect'] := DACMultiDirect(AODI, @DABuf[0], AOChannels);
      //}
      {
      for Ch := 0 to 15 do
        ReportStatus['DACDirect'] := DACDirect(AODI, Ch, Counts)
      ;
      }
    end;
    *)

begin
  if ADCheck.Checked then ADCheck.Checked := False;

  GoButton.Enabled := False;
  AOPanel.Height := 0;
  MainMemo.Lines.Clear;
  Application.ProcessMessages;

  try
    SetLength(CheckChannel, Length(ChannelCheck));
    for Ch := 0 to High(CheckChannel) do
      CheckChannel[Ch] := ChannelCheck[Ch].Checked
    ;

    bAOHasAD := False;
    AOChannels := -1;
    AIDI := diNone;
    AODI := diNone;
    for I := 0 to 31 do begin
      Status := QueryDeviceInfo(I, @PID, nil, nil, nil, nil);
      if Status <> ERROR_SUCCESS then Continue;
      if (PID >= $8040) and (PID < $8060) then
        AIDI := I
      else if (PID >= $8060) and (PID < $8080) then begin
        case PID and $7F86 of
          $0000: AOChannels := 16;
          $0002: AOChannels := 12;
          $0004: AOChannels := 8;
          $0006: AOChannels := 4;
          else Continue; //?!?
        end;
        AODI := I;
        bAOHasAD := (PID and $0001) = 0;
      end;
    end;

    if AIDI = diNone then
      MainMemo.Lines.Add('Failed to find a USB-AI board.');
    if AODI = diNone then
      MainMemo.Lines.Add('Failed to find a USB-AO board.');
    if AOChannels < 0 then
      MainMemo.Lines.Add('Internal error: Unknown USB-AO channels.');

    if (AIDI = diNone) or (AODI = diNone) or (AOChannels < 0) then begin
      MainMemo.Lines.Add('Aborted.');
      exit;
    end;


    AORange := RangeRadios.ItemIndex;
    case AORange of
      0: begin AIRange := 2; {0-5V}  Span :=  5; BiOff := 0;   end;
      1: begin AIRange := 3; {5V}   Span := 10; BiOff := 0.5; end;
      2: begin AIRange := 0; {0-10V} Span := 10; BiOff := 0;   end;
      3: begin AIRange := 1; {10V}  Span := 20; BiOff := 0.5; end;
      else Exit;
    end;

    OffCt := (StrToFloat(OffsetEdit.Text) / Span) * $10000;

    MainMemo.Lines.Add('Configuring, ' + IntToStr(AOChannels) + 'ch...');
    //Set up A/D.
    for Ch := 0 to 15 do Config.ChannelRange[Ch] := AIRange;
    Config.TrigMode := $15; //Scan selected channels each counter rising edge. (Leave counter 0 input set to exposed.)
    Config.StartStopCh := $F0; //Select channels 0 to 15.
    Config.StartStopChEx := $00;
    Config.Oversample := $FF; // 100h oversample.
    ConfigBufSize := SizeOf(Config);
    ReportStatus['ADC_SetConfig'] := ADC_SetConfig(AIDI, @Config, ConfigBufSize);
    Sleep(1);
    ReportStatus['ADC_SetCal'] := ADC_SetCal(AIDI, ':AUTO:');
    //Set up D/A.
    ReportStatus['DACSetBoardRange'] := DACSetBoardRange(AODI, AORange);
    Sleep(1);

    MainMemo.Lines.Add('Loading 1-to-1...');
    TC := GetTickCount;
    for Ch := 0 to AOChannels - 1 do
      if CheckChannel[Ch] then begin
        ReportStatus['DACSetChannelCal(' + IntToStr(Ch) + ')'] := DACSetChannelCal(AODI, Ch, ':1TO1:');
        
        ProcessDelay(InterChannelDelay);
      end
    ;
    TC := GetTickCount - TC;
    MainMemo.Lines.Add('(' + MyFloatToStr(TC / 1000) + 'sec)');

    MainMemo.Lines.Add('Checking one word...');
    SetLength(CalCheck, 1);
    Offset := $AAAA;
    L := Length(CalCheck) * SizeOf(CalCheck[0]);
    CalCheckSize := L;
    ReportStatus['AIOUSB_FlashRead'] := AIOUSB_FlashRead(AODI, Offset*2, CalCheckSize, @CalCheck[0]);
    if CalCheckSize <> L then begin
      MainMemo.Lines.Add('Readback size error, ' + IntToHex(CalCheckSize, 1) + ' vs ' + IntToHex(L, 1));
    end
    else if CalCheck[0] <> Offset then begin
      MainMemo.Lines.Add('Mismatch on cal readback, ' + IntToHex(Offset, 4) + '<>' + IntToHex(CalCheck[0], 4) + '.');
    end;

    MainMemo.Lines.Add('Taking data...');
    ReportStatus['DACMultiDirect'] := SetAllDACs(LoRef);
    Sleep(1);

    for Ch := 0 to AOChannels - 1 do ADLo[Ch] := 0;
    ReportStatus['ADC_GetScanV'] := ADC_GetScanV(AIDI, @ADLo[0]);
    for Ch := 0 to AOChannels - 1 do ADLo[Ch] := (ADLo[Ch]/Span + BiOff) * $10000 + OffCt;
    MainMemo.Lines.Add('Low reference values:');
    MemoBuf := FloatToHex(ADLo[0]);
    for Ch := 1 to AOChannels - 1 do
      MemoBuf := MemoBuf + #9 + FloatToHex(ADLo[Ch])
    ;
    MainMemo.Lines.Add(MemoBuf);

    ReportStatus['DACMultiDirect'] := SetAllDACs(HiRef);
    Sleep(1);

    for Ch := 0 to 15 do ADHi[Ch] := 0;
    ReportStatus['ADC_GetScanV'] := ADC_GetScanV(AIDI, @ADHi[0]);
    for Ch := 0 to AOChannels - 1 do ADHi[Ch] := (ADHi[Ch]/Span + BiOff) * $10000;
    MainMemo.Lines.Add('High reference values:');
    MemoBuf := FloatToHex(ADHi[0]);
    for Ch := 1 to AOChannels - 1 do
      MemoBuf := MemoBuf + #9 + FloatToHex(ADHi[Ch])
    ;
    MainMemo.Lines.Add(MemoBuf);

    for Ch := 0 to AOChannels - 1 do
      if CheckChannel[Ch] then
        if ADHi[Ch] = ADLo[Ch] then
          raise EZeroDivide.Create('High and low readings were both ' + FloatToHex(ADHi[Ch]) + ' for channel ' + IntToHex(Ch, 1) + '.')
    ;

    {
    MainMemo.Lines.Add('Prepping cal file template...');
    SetLength(CalBuf, $70000);
    for I := 0 to $FFFF do begin
      CalBuf[I * 7 + 1] := '$';
      CalBuf[I * 7 + 6] := #13;
      CalBuf[I * 7 + 7] := #10;
    end;
    }
    MainMemo.Lines.Add('Saving cal files...');
    SetLength(CalBuf, $70000);
    for I := 0 to $FFFF do begin
      CalBuf[I * 7 + 1] := '$';
      CalBuf[I * 7 + 6] := #13;
      CalBuf[I * 7 + 7] := #10;
    end;
    for Ch := 0 to AOChannels - 1 do begin
      if CheckChannel[Ch] then begin
        for I := 0 to $FFFF do begin
          TarEntry := (I - ADLo[Ch]) / (ADHi[Ch] - ADLo[Ch]) * (HiRef - LoRef) + LoRef;
          if TarEntry <= $0000 then
            CalEntry := $0000
          else if TarEntry >= $FFFF then
            CalEntry := $FFFF
          else
            CalEntry := Round(TarEntry)
          ;
          CalBuf[I * 7 + 5] := NybbleToHex[CalEntry and $F];
          CalEntry := CalEntry shr 4;
          CalBuf[I * 7 + 4] := NybbleToHex[CalEntry and $F];
          CalEntry := CalEntry shr 4;
          CalBuf[I * 7 + 3] := NybbleToHex[CalEntry and $F];
          CalEntry := CalEntry shr 4;
          CalBuf[I * 7 + 2] := NybbleToHex[CalEntry and $F];
        end;

        FilBuf := 'Cal' + IntToHex(Ch, 1) + '.txt';
        Fil := TFileStream.Create(FilBuf, fmCreate);
        Fil.WriteBuffer(CalBuf[1], Length(CalBuf));
        Fil.Free;
      end;
    end;

    //SetLength(CalCheck, $100);

    MainMemo.Lines.Add('Uploading cal files...');
    TC := GetTickCount;
    for Ch := 0 to AOChannels - 1 do begin
      if CheckChannel[Ch] then begin

        FilBuf := 'Cal' + IntToHex(Ch, 1) + '.txt';
        ReportStatus['DACSetChannelCal(' + IntToStr(Ch) + ')'] := DACSetChannelCal(AODI, Ch, PChar(FilBuf));
        FilBuf := '';

        {
        Offset := $20000*Ch + $2200*Ch; //Offset to this channel, plus offset into this channel, in bytes.
        L := Length(CalCheck) * SizeOf(CalCheck[0]);
        CalCheckSize := L;
        ReportStatus['AIOUSB_FlashRead'] := AIOUSB_FlashRead(AODI, Offset, CalCheckSize, @CalCheck[0]);
        if CalCheckSize <> L then begin
          MainMemo.Lines.Add('Readback error, ' + IntToHex(CalCheckSize, 1) + ' vs ' + IntToHex(L, 1));
          Continue;
        end;
        Offset := $1100*Ch; //Just offset into this channel, in samples.
        try
          for I := 0 to High(CalCheck) do begin
            CalEntry := StrToInt(Copy(CalBuf, (Offset + LongWord(I)) * 7 + 1, 5));
            if CalEntry <> CalCheck[I] then begin
              MainMemo.Lines.Add('Mismatch on cal readback, ' + IntToHex(CalEntry, 4) + '<>' + IntToHex(CalCheck[I], 4) + ' @ ' + IntToHex(Offset + LongWord(I), 1) + ' of ch' + IntToStr(Ch) + '.');
              Break;
            end;
          end;
        except
          MainMemo.Lines.Add('Exception checking cal against file for ch' + IntToStr(Ch) + ' - probably corrupt.');
        end;
        }
      end;
    end;
    TC := GetTickCount - TC;
    MainMemo.Lines.Add('(' + MyFloatToStr(TC / 1000) + 'sec)');



    SetLength(CalBuf, 0);

    ADCheck.Checked := True;

    AOADLabel.Caption := '';
    AOADPanel.Visible := bAOHasAD;
    AOPanel.Height := FullAOHeight;

    DACLowButton.Caption := FloatToStr((1/10 - BiOff) * Span) + 'V';
    DACMidButton.Caption := FloatToStr((5/10 - BiOff) * Span) + 'V';
    DACHighButton.Caption := FloatToStr((9/10 - BiOff) * Span) + 'V';

    MainMemo.Lines.Add('Done.');
    
    Beep;
  finally
    GoButton.Enabled := True;
  end;
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  I, N, iDir: Integer;
  Status: LongWord;
  OutMask: Byte;
  DIOData: Word;
  DIOErr: Byte;
  ErrStr: String;

label
  Done;

begin
  MainMemo.Lines.Clear;
  Application.ProcessMessages;

  if AODI = diNone then begin
    MainMemo.Lines.Add('Need to use Go first.');
    Exit;
  end;

  DIOErr := 0;
  MainMemo.Lines.Add('Testing DIO loopback...');
  OutMask := $01;
  for iDir := 0 to 1 do begin
    DIOData := $FFFF;
    ReportStatus['DIO_Configure'] := DIO_Configure(AODI, False, @OutMask, @DIOData);
    for I := $00 to $FF do begin
      DIOData := $0101 * I;
      Status := DIO_WriteAll(AODI, @DIOData);
      if Status <> ERROR_SUCCESS then begin
        ReportStatus['DIO_WriteAll'] := Status;
        MainMemo.Lines.Add('Aborting DIO loopback test.');
        Exit;
      end;

      DIOData := 0;
      Status := DIO_ReadAll(AODI, @DIOData);
      if Status <> ERROR_SUCCESS then begin
        ReportStatus['DIO_ReadAll'] := Status;
        MainMemo.Lines.Add('Aborting DIO loopback test.');
        Exit;
      end;

      DIOData := DIOData xor ($0101 * I);
      if DIOData <> 0 then DIOErr := DIOErr or DIOData or (DIOData shr 8);
    end;
    OutMask := $02;
  end;

  if DIOErr = 0 then begin
    MainMemo.Lines.Add('DIO loopback: PASS.');
  end
  else begin
    ErrStr := '';
    N := 0;
    for I := 1 to 7 do
      if (DIOErr and (1 shl I)) <> 0 then begin
        if N <> 0 then ErrStr := ErrStr + ', ';
        ErrStr := ErrStr + IntToStr(I) + '(' + IntToHex(1 shl I, 2) + ')';
        Inc(N);
      end
    ;
    if N > 1 then
      ErrStr := 'DIO loopback: Errors in bits ' + ErrStr
    else
      ErrStr := 'DIO loopback: Error in bit ' + ErrStr
    ;
    MainMemo.Lines.Add(ErrStr);
  end;
end;

procedure TMainForm.DoReportStatus(Wot: String; Status: LongWord);
begin
  if Status = ERROR_SUCCESS then
    MainMemo.Lines.Add(Wot + ': OK')
  else
    MainMemo.Lines.Add(Wot + ': Error ' + IntToStr(Status))
  ;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
  Reg: TRegistry;

begin
  AIDI := diNone;
  AODI := diNone;
  QueryPerformanceFrequency(TicksPerSec);
  TicksPerMillisec := TicksPerSec div 1000;
  LoadDefaultFont(Font);
  DelayPanel.Font.Size := Round(3/4 * Font.Size);

  MainMemo.Lines.Text := 'For a single-range board, enter the ground offset(AO ground minus AI ground), select the range, and click Go.'#13#10
                       + #13#10
                       + 'For a board with range jumpers:'#13#10
                       + '* Enter the ground offset(AO ground minus AI ground).'#13#10
                       + '* Select the range the jumpers are configured for, and click Go.'#13#10
                       + '* Wait for it to finish.'#13#10
                       + '* Change the range jumpers and select the new range, and click Go.'#13#10
  ;

  with TLabel.Create(Self) do begin
    AutoSize := False;
    SetBounds(0, 0, 32, 16);
    //Align := alTop;
    Alignment := taCenter;
    Caption := 'Ch';
    Parent := ChannelPanel;
  end;
  for I := 0 to High(ChannelCheck) do begin
    ChannelCheck[I] := TCheckBox.Create(Self);
    with ChannelCheck[I] do begin
      SetBounds(0, 16*I + 16, 32, 16);
      //Align := alTop;
      Checked := True;
      Caption := IntToStr(I);
      Parent := ChannelPanel;
    end;
  end;

  FullADHeight := ADPanel.Height;
  ADPanel.Height := 0;
  ADCheck.Align := alTop;

  FullAOHeight := AOPanel.Height;
  AOPanel.Height := 0;

  Reg := TRegistry.Create(KEY_QUERY_VALUE);
  Reg.RootKey := HKEY_CURRENT_USER;
  if Reg.OpenKey(MyKey, False) then begin
    if Reg.ValueExists('InterChannelDelay') then try
      InterChannelDelay := Reg.ReadInteger('InterChannelDelay');
    except
      InterChannelDelay := 0;
    end;
  end;
  Reg.Free;
  DelayPanel.Caption := ':1TO1: Delay: ' + StrFromSecQuarters(InterChannelDelay div 250);
end;

procedure TMainForm.ADCheckClick(Sender: TObject);
begin
  if ADCheck.Checked then begin
    ADTime.Enabled := True;
    ADPanel.Height := FullADHeight;
  end
  else begin
    ADTime.Enabled := False;
    ADPanel.Height := ADCheck.Height;
  end;
end;

procedure TMainForm.ADTimeTimer(Sender: TObject);
const
  OverOverSample = 7;

var
  I, W, Ch: Integer;
  //K: Double;
  ADData, RawADData: array[0..15] of Double;
  VRect: TRect;

begin
  if AIDI = diNone then begin
    ADCheck.Checked := False;
    Exit;
  end;

  for Ch := 0 to 15 do ADData[Ch] := 0;
  for I := 1 to OverOverSample do begin
    for Ch := 0 to 15 do RawADData[Ch] := 0;
    ADC_GetScanV(AIDI, @RawADData[0]);
    for Ch := 0 to 15 do ADData[Ch] := ADData[Ch] + RawADData[Ch];
  end;
  for Ch := 0 to 15 do ADData[Ch] := 1/OverOverSample * ADData[Ch];
  //for Ch := 0 to 15 do ADData[Ch] := (ADData[Ch]/Span + BiOff) * $10000 + OffCt;

  W := (ADPaint.Width div 4) - 24;
  W := Max(1, W);
  //K := W/$10000;

  ADPaint.Canvas.Font.Name := 'Tahoma';
  ADPaint.Canvas.Font.Color := clBtnText;
  ADPaint.Canvas.Pen.Color := clBtnText;
  //ADPaint.Canvas.Brush.Style := bsClear;
  ADPaint.Canvas.Brush.Color := clBtnFace;
  for Ch := 0 to 15 do begin
    VRect.Top := 16 * (Ch mod 4);
    VRect.Left := (24 + W) * (Ch div 4);
    VRect.Bottom := VRect.Top + 16;
    VRect.Right := VRect.Left + (24 + W);
    //ADPaint.Canvas.Brush.Color := clBtnFace;
    //ADPaint.Canvas.TextOut(X, Y, IntToStr(Ch) + ': ' + IntToHex(Round(ADData[Ch]), 4) + ' ');
    //ADPaint.Canvas.TextOut(X, Y, IntToStr(Ch) + ': ' + MyFloatToStr(ADData[Ch]) + ' ');
    ADPaint.Canvas.TextRect(VRect, VRect.Left, VRect.Top, IntToStr(Ch) + ': ' + MyFloatToStr(ADData[Ch]));
    {
    Inc(X, 16);
    X2 := Round(X + K * ADData[Ch]);
    ADPaint.Canvas.Brush.Color := clBtnText;
    ADPaint.Canvas.FillRect(Rect(X, Y + 4, X2, Y + 12));
    ADPaint.Canvas.Brush.Color := clBtnFace;
    ADPaint.Canvas.FillRect(Rect(X2, Y + 4, X + W, Y + 12));
    ADPaint.Canvas.Brush.Style := bsClear;
    ADPaint.Canvas.Rectangle(X, Y + 4, X + W, Y + 12);
    }
  end;



  if (AODI <> diNone) and AOADPanel.Visible then begin
    I := 1 shl SmoothTrack.Position;
    ADData[0] := 0;
    ADData[1] := 0;
    for I := 1 to I do begin
      RawADData[0] := 0;
      RawADData[1] := 0;
      ADC_GetScanV(AODI, @ADData[0]);
      ADData[0] := ADData[0] + RawADData[0];
      ADData[1] := ADData[1] + RawADData[1];
    end;

    AOADLabel.Caption := '0: ' + MyFloatToStr(ADData[0]) + #13#10'1: ' + MyFloatToStr(ADData[1]);
  end;
end;

procedure TMainForm.DACLowButtonClick(Sender: TObject);
begin
  ReportStatus['DACMultiDirect'] := SetAllDACs(6554);
end;

procedure TMainForm.DACMidButtonClick(Sender: TObject);
begin
  ReportStatus['DACMultiDirect'] := SetAllDACs(32768);
end;

procedure TMainForm.DACHighButtonClick(Sender: TObject);
begin
  ReportStatus['DACMultiDirect'] := SetAllDACs(58982);
end;

procedure TMainForm.DelayPanelClick(Sender: TObject);
var
  DelayForm: TForm;
  BlurbLabel, DelayLabel: TLabel;
  DelayTrack: TTrackBar;
  BorderPanel: TPanel;
  OKButton: TButton;
  CancelButton: TButton;
  Reg: TRegistry;

  procedure NewBorderPanel;
  begin
    BorderPanel := TPanel.Create(DelayForm);
    BorderPanel.Top := 9001;
    BorderPanel.Align := alBottom;
    BorderPanel.BevelOuter := bvNone;
    BorderPanel.BorderWidth := 4;
    BorderPanel.Parent := DelayForm;
  end;

begin
  DelayForm := TForm.Create(Self);
  LoadDefaultFont(DelayForm.Font);
  DelayForm.Position := poMainFormCenter;
  DelayForm.Caption := ':1TO1: Delay';
  DelayForm.ClientWidth := 256;
  DelayForm.ClientHeight := 128;
  DelayForm.BorderWidth := 4;

  BlurbLabel := TLabel.Create(DelayForm);
  BlurbLabel.AutoSize := False;
  BlurbLabel.Align := alClient;
  BlurbLabel.WordWrap := True;
  BlurbLabel.Alignment := taCenter;
  BlurbLabel.Layout := tlCenter;
  BlurbLabel.Parent := DelayForm;

  NewBorderPanel;
    DelayLabel := TLabel.Create(DelayForm);
    DelayLabel.AutoSize := False;
    DelayLabel.Align := alTop;
    DelayLabel.Parent := BorderPanel;
    DelayLabel.Name := 'DelayLabel';

    DelayTrack := TTrackBar.Create(DelayForm);
    DelayTrack.SetBounds(0, 9001, 192, 24);
    DelayTrack.ThumbLength := 15;
    DelayTrack.Align := alTop;
    DelayTrack.Min := 0;
    DelayTrack.Max := 16*4;
    DelayTrack.Position := InterChannelDelay div 250;
    DelayTrack.PageSize := 4;
    DelayTrack.OnChange := DelayTrackChange;
    DelayTrack.Parent := BorderPanel;
  BorderPanel.AutoSize := True;

  NewBorderPanel;
    OKButton := TButton.Create(DelayForm);
    OKButton.Caption := 'OK';
    OKButton.Default := True;
    OKButton.ModalResult := mrOk;
    OKButton.Align := alLeft;
    OKButton.Parent := BorderPanel;

    CancelButton := TButton.Create(DelayForm);
    CancelButton.Caption := 'Cancel';
    CancelButton.Default := True;
    CancelButton.ModalResult := mrCancel;
    CancelButton.Align := alRight;
    CancelButton.Parent := BorderPanel;
  BorderPanel.Height := 32;

  BlurbLabel.Caption := 'This delay is used to allow each :1TO1: cal write to "settle".';
  DelayTrackChange(DelayTrack);
  if DelayForm.ShowModal = mrOk then begin
    InterChannelDelay := DelayTrack.Position * 250;
    DelayPanel.Caption := ':1TO1: Delay: ' + StrFromSecQuarters(InterChannelDelay div 250);

    Reg := TRegistry.Create(KEY_CREATE_SUB_KEY or KEY_SET_VALUE);
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(MyKey, True) then
      Reg.WriteInteger('InterChannelDelay', InterChannelDelay)
    ;
    Reg.Free;
  end;

  DelayForm.Free;
end;

procedure TMainForm.DelayTrackChange(Sender: TObject);
var
  DelayLabel: TLabel;
  DelayTrack: TTrackBar;

begin
  DelayTrack := TTrackBar(Sender);
  DelayLabel := TLabel(FindChildControlAndIMeanControl(DelayTrack.Parent, 'DelayLabel'));

  DelayLabel.Caption := StrFromSecQuarters(DelayTrack.Position);
end;

end.
